Rem ------------------------------------------------------------------
Everything HUD related
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''


Rem ------------------------------------------------------------------
Draws the HUD with all components
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Function DrawHUD()
	' stats
	Local health:Int, healthmax:Int
	Local mana:Int, manamax:Int
	Local energy:Int, energymax:Int
	' defaults
	If Not player then
		healthmax	= FIG_PLR_SETTINGS[2]
		health		= healthmax
		manamax		= FIG_PLR_SETTINGS[4]
		mana		= manamax
		energymax	= FIG_PLR_SETTINGS[6]
		energy		= energymax
	' player stats
	Else
		healthmax	= player.healthmax
		health		= player.health
		manamax		= player.manamax
		mana		= player.mana
		energymax	= player.energymax
		energy		= player.energy
	EndIf
	' gains must be tweaked here, not part of spell type
	Local gainh:Int, gainm:Int, gaine:Int
	If selectedSpell = 5 then
		gainh = 400000
	ElseIf selectedSpell = 6
		gainm = 400000
	EndIf
	' stats
	SetBlend(ALPHABLEND)
	SetAlpha(1)
	SetColor(255,255,255)
	DrawImage(imgHUD, 0, 640)
	SetColor(0,255,0)
	DrawHUDBar(48, 672, health, SPELLS[selectedSpell].health, gainh, 0, healthmax)
	SetColor(0,170,255)
	DrawHUDBar(48, 696, mana, SPELLS[selectedSpell].mana, gainm, 0, manamax)
	SetColor(255,170,0)
	DrawHUDBar(48, 720, energy, 0, gaine, SPELLS[selectedSpell].energy, energymax)
	' icons
	Local iconstate:Int, iconstatedata:Int, cnt:Int=-1
	For Local i:Int = 0 Until 8
		' defaults
		If Not player then
			iconstate = ICONSTATE_AVAILABLE
			If i >= 5 then cnt = 5
		' player
		Else
			If i >= 5 then
				cnt = player.itemcount[i-5]
			EndIf
			' spells avail
			If player.mana < SPELLS[i].mana then
				iconstate = ICONSTATE_MANA
				iconstatedata = 64* (SPELLS[i].mana -player.mana) /Float(player.manamax)
			ElseIf player.energy < SPELLS[i].energy then
				iconstate = ICONSTATE_ENERGY
			Else
				iconstate = ICONSTATE_AVAILABLE
				' items avail
				If i >= 5 And cnt = 0 then
					iconstate = ICONSTATE_ENERGY
				EndIf
			EndIf
		EndIf
		DrawHUDIcon(SPELLS[i].icon, 432+i*72 +8*(i>4), 672, iconstate, (i=selectedSpell), iconstatedata, cnt, SPELLS_HOTKEYS_CAPTIONS[i])
	Next
	' buffs
	If player And player.buff <> BUFF_NONE then
		Local frame:Int
		If player.buff = BUFF_FREEZE then frame = ICON_FROZEN
		If player.buff = BUFF_WEAK then frame = ICON_WEAK
		DrawHUDIcon(frame, 320, 672, ICONSTATE_AVAILABLE)
	EndIf
	' tooltips
	SetBlend(ALPHABLEND)
	SetAlpha(1)
	SetColor(255,255,0)
	fntSmall.Draw(SPELLS[selectedSpell].tooltip, SCREEN_WIDTH/2, SCREEN_HEIGHT-5, ALIGN_CENTER)
End Function

Rem ------------------------------------------------------------------
Draw a stat bar on the HUD with base value, costs and gains
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Function DrawHUDBar(pX:Int, pY:Int, pBase:Int, pCost:Int, pGain:Int, pReqs:Int, pMaxVal:Int)
	Local scaler:Float = 256.0 / Float(pMaxVal)
	pBase :* scaler
	pCost :* scaler
	pGain :* scaler
	pReqs :* scaler
	pMaxVal :* scaler
	' prepare stats
	Local base:Int = pBase
	Local costl:Int = Max(base -pCost, 0)
	Local costu:Int = base
	Local cost:Int = costu -costl
	Local gainl:Int = base
	Local gainu:Int = Min(base +pGain, pMaxVal)
	Local gain:Int = gainu -gainl
	Local reqs:Int = Min(base, pReqs)
	' draw
	SetAlpha(1)
	DrawSubImageRect(imgBar, pX			, pY, base, 16, 0		, 0, base, 16, 0,0, 0)
	DrawSubImageRect(imgBar, pX+costl	, pY, cost, 16, costl	, 0, cost, 16, 0,0, 2)
	DrawSubImageRect(imgBar, pX			, pY, reqs, 16, reqs	, 0, reqs, 16, 0,0, 2)
	SetAlpha(0.25)
	DrawSubImageRect(imgBar, pX+gainl	, pY, gain, 16, gainl	, 0, gain, 16, 0,0, 0)
End Function

Rem ------------------------------------------------------------------
Draw HUD icon with availability and highlight
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Function DrawHUDIcon(pIcon:Int, pX:Int, pY:Int, pIconState:Int, pSelected:Int=0, pIconstateData:Int=0, pCount:Int=-1, pHotkey:String="")
	' icon
	SetBlend ALPHABLEND
	Select pIconState
	Case ICONSTATE_AVAILABLE
		SetAlpha(1)
		SetColor(56,59,61)
		DrawRect(pX, pY, 64, 64)
		SetColor(255,255,255)
		DrawImage(imgIcons, pX, pY, pIcon)
	Case ICONSTATE_ENERGY
		SetAlpha(1)
		SetColor(112,118,122)
		DrawRect(pX, pY, 64, 64)
		SetAlpha(0.15)
		SetColor(255,255,255)
		DrawImage(imgIcons, pX, pY, pIcon)
	Case ICONSTATE_MANA
		SetAlpha(0.5)
		SetColor(187,187,187)
		DrawRect(pX, pY, 64, 64)
		SetColor(0,85,128)
		DrawRect(pX, pY+64, 64, -64+pIconstateData)
		SetColor(255,255,255)
		DrawImage(imgIcons, pX, pY, pIcon)
	End Select
	' count / hotkey
	SetAlpha(1)
	SetColor(255,255,255)
	fntSmall.Draw(pHotkey, pX+4, pY+4, ALIGN_LEFT, ALIGN_TOP)
	If pCount >= 0 then
		fntBig.Draw(pCount, pX+60,pY+60, ALIGN_RIGHT)
	EndIf
	' highlight
	If pSelected then
		SetBlend(LIGHTBLEND)
		SetAlpha(1)
		SetColor(255,255,0)
		DrawImage(imgIconSelector, pX, pY)
	EndIf
End Function


Rem ------------------------------------------------------------------
Draw a button, return true if hit
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Function DrawButton:Int(pTxt:String, pX:Int, pY:Int)
	Local ret:Int = False
	' background
	SetAlpha(1)
	SetColor(255,255,255)
	DrawImage(imgButton, pX -BUTTON_WIDTH/2, pY -BUTTON_HEIGHT/2)
	' hover
	If mx >= pX-BUTTON_WIDTH/2 And mx < pX+BUTTON_WIDTH/2 And my >= pY-BUTTON_HEIGHT/2 And my < pY+BUTTON_HEIGHT/2 then
		SetColor(255,255,255)
		If mh1 then
			ret = True
		EndIf
	Else
		SetColor(255,255,0)
	EndIf
	' caption
	fntBig.Draw(pTxt, pX, pY, ALIGN_CENTER, ALIGN_CENTER)
	Return ret
End Function


Rem ------------------------------------------------------------------
Draw mouse pointer
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Function DrawCursor()
	SetBlend(ALPHABLEND)
	SetAlpha(1)
	SetColor(COL_POINTER[0], COL_POINTER[1], COL_POINTER[2])
	DrawImage(imgCursor, mx, my)
End Function
