Rem ------------------------------------------------------------------
Everything figure related
AI
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''


Rem ------------------------------------------------------------------
Figure. All creatures, player and enemy, are of this type.
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Type TFigure
	Global list:TList = CreateList()
	
	Field skin:Int
	Field radius:Float = 8
	
	Field posx:Float, posy:Float
	Field rotation:Float
	Field speed:Float = 2
	Field obstacle:TLightObstacle
	
	' life attributes, ppm for practical reasons
	Field health:Int, healthmax:Int, healthreg:Int
	Field mana:Int, manamax:Int, manareg:Int
	Field energy:Int, energymax:Int, energyreg:Int
	Field buff:Int
	Field itemcount:Int[3]

	Method New()
		list.AddLast(Self)
	End Method
	
	Method Init(pSkin:Int, pSettings:Int[], pX:Int, pY:Int)
		skin		= pSkin
		radius		= pSettings[0]
		speed		= pSettings[1] / 100.0
		healthmax	= pSettings[2]
		health		= healthmax
		healthreg	= pSettings[3]
		manamax		= pSettings[4]
		mana		= manamax
		manareg		= pSettings[5]
		energymax	= pSettings[6]
		energy		= energymax
		energyreg	= pSettings[7]
		itemcount[0]	= pSettings[8]
		itemcount[1]	= pSettings[9]
		itemcount[2]	= pSettings[10]
		posx		= pX
		posy		= pY
		obstacle	= New TLightObstacle.Init(posx, posy, radius, imgFiguresOverlay, skin)
	End Method
	
	' update all figures
	Function UpdateAll()
		For Local fig:TFigure = EachIn list
			fig.Update()
		Next
	End Function
	
	Method Update()
		Local mreg:Int = manareg
		Local ereg:Int = energyreg
		If buff = BUFF_WEAK then
			mreg :* 0.5
			ereg :* 0.125
		EndIf
		health	= Min(health +healthreg, healthmax)
		mana	= Min(mana +mreg, manamax)
		energy	= Min(energy +ereg, energymax)
	End Method
	
	' move
	' along the four possible axes, forward, backward, left, right
	' or north, south, east, west
	Method Move(pFw:Int, pBk:Int, pLf:Int, pRt:Int, pControls:Int = CONTROL_RELATIVE)
		' don't move if frozen
		If buff = BUFF_FREEZE then
			New TParticle.Init(posx,posy, 0,0, Rnd(radius *0.4, radius *0.8), COL_FREEZE, 50, 0.2)
			Return
		EndIf
		' direction of movement
		Local moveu:Float, movev:Float, movea:Float
		moveu = pFw -pBk
		movev = pRt -pLf
		' only move if really moving
		If moveu <> 0 Or movev <> 0 then
			movea = atan2(movev, moveu)
			' direction of movement in player space and absolute
			Local mrela:Float, mabsa:Float
			If pControls = CONTROL_ABSOLUTE then
				movea = (movea +270) Mod 360
				mrela = GetAngleDiff(rotation, movea)
				mabsa = movea
			Else
				mrela = movea
				mabsa = rotation +mrela
			EndIf
			' speed depends on relative movement direction
			Local movespeed:Float
			movespeed = 1 -(FIG_SPEED_NORMAL -FIG_SPEED_BACK) *mrela *mrela / (180.0*180.0)
			' move in world
			Local nx:Float, ny:Float
			nx = posx +cos(mabsa)*movespeed
			ny = posy +sin(mabsa)*movespeed
			' boundaries
			If nx < MAP_BORDER_LF then nx = MAP_BORDER_LF
			If nx > MAP_BORDER_RT then nx = MAP_BORDER_RT
			If ny < MAP_BORDER_TP then ny = MAP_BORDER_TP
			If ny > MAP_BORDER_BT then ny = MAP_BORDER_BT
			' other figures (earlier created figures have priority)
			For Local fig:TFigure = EachIn TFigure.list
				If fig <> Self then
					Local dx:Float, dy:Float, d:Float
					dx = fig.posx -nx
					dy = fig.posy -ny
					d = sqr(dx*dx +dy*dy)
					If d < (radius +fig.radius +5) then
						nx = posx
						ny = posy
					EndIf
				EndIf
			Next
			' adapt position
			posx = nx
			posy = ny
			' gfx
			obstacle.x = posx
			obstacle.y = posy
		EndIf
		If buff = BUFF_WEAK then New TParticle.Init(posx,posy, 0,0, Rnd(radius *0.4, radius *0.8), COL_WEAKEN, 50, 0.2)
	End Method

	' draw all figures
	Function DrawAll()
		' draw skin
		For Local fig:TFigure = EachIn list
			fig.Draw()
		Next
		SetRotation(0)
		' draw UI overlays
		For Local fig:TFigure = EachIn list
			fig.DrawUI()
		Next
		TLightObstacle.DrawAll()
	End Function

	Method Draw()
		SetBlend(ALPHABLEND)
		SetAlpha(1)
		SetColor(255,255,255)
		SetRotation(rotation)
		DrawImage(imgFigures, posx, posy, skin)
	End Method
	
	Method DrawUI()
	
	End Method
End Type

Rem ------------------------------------------------------------------
Human controlled figure
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Type TFigureHuman Extends TFigure
	Method Update()
		Super.Update()
		Local dmx:Float, dmy:Float
		dmx = mx -posx
		dmy = my -posy
		rotation = atan2(dmy, dmx)
		' move around
		Move(kdW, kdS, kdA, kdD, controls)
		' attacking
		If mh1 then
			SPELLS[selectedSpell].Cast(player)
		EndIf
		' hotkeys
		For Local i:Int = 0 Until 8
			If khHot[i] then
				SPELLS[i].Cast(player)
			EndIf
		Next
	End Method
End Type

Rem ------------------------------------------------------------------
AI controlled figure
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Type TFigureAI Extends TFigure
	Field spawnpoint:TFigSpawnpoint
	
	Field target:TFigure

	' calculating probability of scenarios
	Field prob_limits:Float[9], prob_max:Float, prob_idx:Int

	Method Update()
		Super.Update()
		' die
		If health <= 0 then
			' initiate respawn
			If spawnpoint then
				spawnpoint.count :- 1
				If spawnpoint.count = 0 then
					spawnpoint.nexttime = ms +RESPAWN_TIME
				EndIf
			EndIf
			' drop
			New TDrop.Init(posx, posy, Rand(0,2), Rand(1,5))
			list.Remove(Self)
			obstacle.list.Remove(obstacle)
		EndIf
		' observe
		Local hitx:Float, hity:Float, hitdist:Float, gettinghit:Int = False
		' check if any projectile is going to hit me
		For Local proj:TProjectile = EachIn TProjectile.list
			' yep, dodge
			If proj.prefigure = Self then
				Local hitpoint:Float[2], hdist:Float
				hitx = proj.prepoint[0] -posx
				hity = proj.prepoint[1] -posy
				hitdist = Normalize(hitx, hity)
				gettinghit = True
				' and fucking aggro all of squad
				If Not target And proj.figure = player then
					' whole squad
					If spawnpoint then
						For Local figs:TFigureAI = EachIn spawnpoint.spawns
							figs.target = player
						Next
					' just me
					Else
						target = player
					EndIf
				EndIf
				Exit
			EndIf
		Next
		' coordinates to player (target)
		Local tarx:Float, tary:Float, tardist:Float
		Local ffpx:Float, ffpy:Float, ffpdist:Float
		Local mpcx:Float, mpcy:Float, mpcdist:Float, mpcf:Float
		If target then
			tarx = target.posx -posx
			tary = target.posy -posy
			tardist = Normalize(tarx, tary)
			' check if I'd be going to friendly fire
			Local collplayer:TFigure, collpos:Float[2]
			collplayer = GetNearestTouching(posx,posy, tarx,tary, player.radius, collpos)
			If collplayer <> player And collplayer <> Null then
				ffpx = collpos[0] -collplayer.posx
				ffpy = collpos[1] -collplayer.posy
				ffpdist = Normalize(ffpx, ffpy)
			EndIf
			' move towards map centre
			mpcx = SCREEN_WIDTH/2 -posx
			mpcy = SCREEN_HEIGHT/2 -posy
			mpcdist = Normalize(mpcx, mpcy)
			mpcf = Max(0, Min(1, (mpcdist / 272.0) ^4 -0.5))
		EndIf
		' define point to move to
		Local movx:Float, movy:Float
		Local tarsgn:Int
		If tardist > 200 then
			tarsgn = 1
		ElseIf tardist < 140 then
			tarsgn = -1
		EndIf
		movx = -10 *hitx +4 *ffpx +tarx *tarsgn +mpcx*mpcf
		movy = -10 *hity +4 *ffpy +tary *tarsgn +mpcy*mpcf

		If target then
			' turn in target direction
			rotation = atan2(tary, tarx)
		EndIf
		' move to point, det moveto-point in player local coordinate frame
		' local facing vector and local orthogonal vector
		Local lfx:Float, lfy:Float, lox:Float, loy:Float
		lfx = cos(rotation)
		lfy = sin(rotation)
		lox = lfy
		loy = -lfx
		' project moveto-point coordinates onto local frame axes
		Local df:Float, do:Float
		df = lfx*movx +lfy*movy
		do = lox*movx +loy*movy
		' move
		Move(df>0.1, df<-0.1, do>0.2, do<-0.2)
		
		' attack / act
		If target then
			' some actions should only be taken if trajectory is free
			Local collplayer:TFigure, collpos:Float[2]
			Local mayshoot:Int = 1
			tarx = target.posx -posx
			tary = target.posy -posy
			tardist = Normalize(tarx, tary)
			collplayer = GetNearestTouching(posx,posy, tarx,tary, 3, collpos)
			If collplayer <> target then
				mayshoot = 0
			EndIf
			' add probability for scenarios
			ResetScenarios()
			' rate scenarios
			' scenarios are enumerated in order of adding
			' attack
			AddScenario( SPELL_ATTACK.Avail(Self) *Gauss(tardist -150, 50) *mayshoot )
			' heavy
			AddScenario( SPELL_HEAVY.Avail(Self) *InvRamp(tardist, 100 +300 *(target.buff = BUFF_FREEZE)) *50 *mayshoot )
			' bomb
			AddScenario( 0 )
			' freeze
			AddScenario( SPELL_FREEZE.Avail(Self) *Gauss(tardist -200, 150) * (target.buff = BUFF_NONE) *mayshoot )
			' weaken
			AddScenario( SPELL_WEAKEN.Avail(Self) *Gauss(tardist -250, 200) * (target.buff = BUFF_NONE) *mayshoot )
			' heal
			AddScenario( SPELL_HEAL.Avail(Self) *InvRamp(health, healthmax *0.5) )
			' mana reg
			AddScenario( SPELL_MANAREG.Avail(Self) *InvRamp(mana, manamax *0.3) )
			' debuff
			AddScenario( SPELL_DEBUFF.Avail(Self) *(buff <> BUFF_NONE) )
			' idle
			AddScenario(ai_idle_value)
			
			Local scen:Int = ChooseScenario()
			Select scen
				Case 0
					SPELL_ATTACK.Cast(Self)
				Case 1
					SPELL_HEAVY.Cast(Self)
				Case 2
				Case 3
					SPELL_FREEZE.Cast(Self)
				Case 4
					SPELL_WEAKEN.Cast(Self)
				Case 5
					SPELL_HEAL.Cast(Self)
				Case 6
					SPELL_MANAREG.Cast(Self)
				Case 7
					SPELL_DEBUFF.Cast(Self)
			End Select
		EndIf
	End Method

	Method DrawUI()
		SetColor(0,255,0)
		Local h:Int = 2*radius *(Float(health) /Float(healthmax))
		DrawRect(posx-radius, posy-radius-10, h, 4)
	End Method
	
	Method ResetScenarios()
		prob_max = 0
		prob_idx = 0
		For Local i:Int = 0 Until 9
			prob_limits[i] = 0
		Next
	End Method

	Method AddScenario(pProbValue:Float)
		prob_limits[prob_idx] = prob_max +pProbValue
		prob_max :+ pProbValue
		prob_idx :+ 1
	End Method

	Method ChooseScenario:Int()
		Local val:Float = Rnd(0, prob_max)
		For Local i:Int = 0 Until 9
			If val <= prob_limits[i] then
				Return i
			EndIf
		Next
	End Method

End Type