Rem ------------------------------------------------------------------
Simple bitmap font type
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Const ALIGN_LEFT:Int	= 0
Const ALIGN_CENTER:Int	= 1
Const ALIGN_RIGHT:Int	= 2
Const ALIGN_TOP:Int		= 0
Const ALIGN_BASELINE:Int= 2
' image font, simplified
Type TFont
	Field image:TImage, cellsize:Int
	Field offset_x:Int, offset_y:Int, spacing:Int, fontheight:Int
	Field widths:Int[] = New Int[256]
	
	' loads a font, pName.png and pName.dat must exist!
	Method Init:TFont(pName:String)
		' first: read font info (.dat) file, containing char widths, cell size and x/y offset
		Local file:TStream = ReadFile(pName +".dat")
		Local char:Byte, buffer:String, value:Int
		Local eot:Byte=False ' used to indicate end of text (white space or file ending)
		Local idx:Int = 0
		' read as long as there is data in file
		While Not file.Eof()
			char = file.ReadByte()
			' numerical char 48 == ascii:0, 57 == ascii:9
			If char >= 48 And char <= 57 then
				' add char to buffer
				buffer :+ Chr(char)
				eot = False
			Else
				eot = True
			EndIf
			' set end of text at end of file
			If file.Eof() then eot=True
			' any other char
			If eot = True And buffer <> "" then
				' read integer value from buffer
				value = Int(buffer)
				' depending on idx, interpret
				Rem
					0...255		width
					256			cellsize
					257,258		x/y-offset
					259			spacing
					260			font height
				end Rem
				If idx <= 255 then
					widths[idx] = value
				ElseIf idx = 256 then
					cellsize = value
				ElseIf idx = 257 then
					offset_x = value
				ElseIf idx = 258 then
					offset_y = value
				ElseIf idx = 259 then
					spacing = value
				ElseIf idx = 260 then
					fontheight = value
				EndIf
				' get ready for next value
				buffer = ""
				idx :+ 1
			EndIf
		Wend
		file.Close()
		' second: load image file
		image = LoadAnimImage(pName +".png", cellsize, cellsize, 0, 256)
		Return self
	End Method
	
	' draw a text
	Method Draw(pText:String, pX:Int, pY:int, pHorizontalAlign:Int=0, pVerticalAlign:Int=ALIGN_BASELINE)
		' ^ due to possible scaling, use float for position
		' save active scaling
		Local scale_x:Float, scale_y:Float
		GetScale(scale_x, scale_y)
		' remove offset_x (aka the white space left of each char)
		pX :- offset_x *scale_x
		' position according to horizontal aligning
		If pHorizontalAlign = ALIGN_CENTER
			pX :- GetWidth(pText) *0.5 *scale_x
		ElseIf pHorizontalAlign = ALIGN_RIGHT
			pX :- GetWidth(pText) *scale_x
		EndIf
		' remove offset_y
		pY :- offset_y *scale_y
		' position according to vertical aligning
		If pVerticalAlign = ALIGN_CENTER
			pY :+ GetHeight(pText) *0.5 *scale_y
		ElseIf pVerticalAlign = ALIGN_TOP
			pY :+ GetHeight(pText) *scale_y
		EndIf
		' draw text char by char
		Local char:Byte
		For Local i:Int = 0 Until pText.length
			char = pText[i]
			' draw only valid chars
			If char <= 255 then
				DrawImage(image, pX, pY, char)
				' move drawing position
				pX :+ (widths[char] -offset_x +spacing) *scale_x
			EndIf
		Next
	End Method
	
	' return width of given text
	Method GetWidth:Float(pText:String)
		Local width:Float
		' sum up widths char by char
		Local char:Byte
		For Local i:Int = 0 Until pText.length
			char = pText[i]
			If char <= 255 then
				width :+ (widths[char] -offset_x +spacing)
			EndIf
		Next
		' subtract spacing once, for last char of text
		width :- spacing
		Return width
	End Method
	
	' return height of given text
	' function only exists for consistency
	Method GetHeight:Float(pText:String)
		Return fontheight
	End Method
End Type
