Global LIGHT_COL_BLACK_3F:Float[] = [0#, 0#, 0#]
Global LIGHT_STRUCTURE:TImage
Global LIGHT_STRUCTW:Float, LIGHT_STRUCTH:Float

Rem ------------------------------------------------------------------
Set the structure which will be used for drawing
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Function Light_SetStructure(pImage:TImage)
	LIGHT_STRUCTURE	= pImage
	LIGHT_STRUCTW	= LIGHT_STRUCTURE.width
	LIGHT_STRUCTH	= LIGHT_STRUCTURE.width
End Function

Rem ------------------------------------------------------------------
Lightsource
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Type TLightSource
	Global list:TList = New TList
	' position
	Field x:Float, y:Float
	' lighting information
	Field radius:Float, color:Float[3]
	' light geometry
	Field geometry:TList
	' obstacles
	Field obstacles:TList
	
Rem ------------------------------------------------------------------
Initialize new light source. pColor is int[3] ranging 0...255
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
	Method Init:TLightSource(pRadius:Float, pColor:Int[])
		radius		= pRadius
		' copy color components
		For Local i:Int = 0 Until 3
			color[i]	= pColor[i] /255.0
		Next
		list.AddLast(Self)
		Return Self
	End Method

	Function UpdateAll()
		For Local light:TLightSource = EachIn list
			light.UpdateGeometry()
		Next
	End Function

Rem ------------------------------------------------------------------
Build lighting geometry (respecting obstacles)
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
	Method UpdateGeometry()
		' collect obstacles
		Local obstacles:TLightObstacle[] = New TLightObstacle[0]
		For Local o:TLightObstacle = EachIn TLightObstacle.list
			Local dx:Float, dy:Float, d:Float
			dx = o.x -x
			dy = o.y -y
			d = sqr(dx*dx +dy*dy)
			If d < radius then
				obstacles :+ [o]
			EndIf
		Next
		' generate geometry (lightning parts)
		geometry = New TList
		Local a:Int
		For Local pc:Int = 0 Until 24
			a = pc * 15
			AddGeometry(a, a+15, radius, 0, obstacles, 0)
		Next
	End Method
	
	Method AddGeometry(pAngle0:Float, pAngle1:Float, pDist:Float, pAlpha:Float, pObstacles:TLightObstacle[], pIObstacle:Int)
		' no obstacle left, add full geometry
		If pIObstacle >= pObstacles.length then
			geometry.AddLast( New TLightGeometry.Init(x, y, pAngle0, pAngle1, pDist, pDist, pAlpha, pAlpha) )
		' obstacles left, check
		Else
			Local obs:TLightObstacle = pObstacles[pIObstacle]
			' get coordinates to obstacle
			Local dx:Float, dy:Float, d:Float, alpha:Float, beta:Float, b0:Float, b1:Float
			dx = obs.x -x
			dy = obs.y -y
			d = sqr(dx*dx +dy*dy)
			alpha = (atan2(dy, dx) +360) Mod 360
			beta = asin(obs.r / d)
			b0 = (alpha -beta +360) Mod 360
			b1 = (alpha +beta) Mod 360
			' obstacle whithin range
			If d < pDist And GetAngleDiff(pAngle0,b1)>0 And GetAngleDiff(pAngle1,b0)<0 then
				If GetAngleDiff(pAngle0,b0)>0 then
					AddGeometry(pAngle0, b0, pDist, pAlpha, pObstacles, pIObstacle +1)
				EndIf
				Local as:Float, ae:Float, a:Float
				as = MostPosAngle(pAngle0, b0)
				ae = MostNegAngle(pAngle1, b1)
				Local dtang:Float = d *cos(beta)
				a = 1- (1- pAlpha) *dtang/pDist
				'geometry.AddLast( New TLightGeometry.Init(x, y, as, ae, dtang, dtang, a, a) )
				AddGeometry(as, ae, dtang, a, pObstacles, pIObstacle +1)
				If GetAngleDiff(pAngle1,b1)<0 then
					AddGeometry(b1, pAngle1, pDist, pAlpha, pObstacles, pIObstacle +1)
				EndIf
			Else
				'geometry.AddLast( New TLightGeometry.Init(x, y, pAngle0, pAngle1, radius, radius, 0, 0) )
				AddGeometry(pAngle0, pAngle1, pDist, pAlpha, pObstacles, pIObstacle +1)
			EndIf

		EndIf
		'geometry.AddLast( New TLightGeometry.Init(x, y, pAngle0, pAngle1, radius, radius, 0, 0) )
	End Method

	Function DrawAll()
		For Local light:TLightSource = EachIn list
			light.DrawShadow()
		Next
		For Local light:TLightSource = EachIn list
			light.DrawLight()
		Next
		For Local light:TLightSource = EachIn list
			light.DrawHilight()
		Next
		glenable GL_TEXTURE_2D
	End Function

Rem ------------------------------------------------------------------
Draw shadow part of source
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
	Method DrawShadow()
		Local iframe:TImageFrame = LIGHT_STRUCTURE.Frame(0)
		If iframe then
			SetBlend(ALPHABLEND)
			Local scale:Float = SCALE_SHADOW
			' dirty hack!
			' necessary to set state_boundtex
			' also, this enables GL_TEXTURE_2D and sets the texture
			iframe.Draw(0,0 ,0,0 ,0,0 ,0,0 ,0,0)
			' begin drawing triangles
			glbegin GL_TRIANGLES
			For Local g:TLightGeometry = EachIn geometry				
				g.Draw(scale, LIGHT_COL_BLACK_3F, 0.4)
			Next
			glEnd
		EndIf
	End Method
	
Rem ------------------------------------------------------------------
Draw light part of source
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
	Method DrawLight()
		Local iframe:TImageFrame = LIGHT_STRUCTURE.Frame(0)
		If iframe then
			SetBlend(LIGHTBLEND)
			Local scale:Float = SCALE_LIGHT
			' dirty hack!
			' necessary to set state_boundtex
			' also, this enables GL_TEXTURE_2D and sets the texture
			iframe.Draw(0,0 ,0,0 ,0,0 ,0,0 ,0,0)
			glenable GL_TEXTURE_2D
			' begin drawing triangles
			glbegin GL_TRIANGLES
			Local camb:Float[3]
			For Local i:Int = 0 Until 3
				camb[i] = LIGHT_COL_AMBIENT[i] /255.0 *0.5
			Next
			For Local g:TLightGeometry = EachIn geometry			
				g.Draw(scale, camb, 0.8)
				g.Draw(scale, color, 0.5)
			Next
			glEnd
		EndIf
	End Method
	
Rem ------------------------------------------------------------------
draw hilight part of source
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
	Method DrawHilight()
		SetBlend(LIGHTBLEND)
		' begin drawing triangles
		gldisable GL_TEXTURE_2D
		glbegin GL_TRIANGLES
		Local camb:Float[3]
		For Local i:Int = 0 Until 3
			camb[i] = color[i] *0.2
		Next
		For Local g:TLightGeometry = EachIn geometry			
			g.Draw(1, camb, 0.1)
		Next
		glEnd
	End Method
End Type

Rem ------------------------------------------------------------------
Obstacles that block light
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Type TLightObstacle
	Global list:TList = New TList
	Field r:Float
	Field x:Float, y:Float
	Field img_overlay:TImage, img_overlay_frame:Int
	
	Method Init:TLightObstacle(pX:Int, pY:Int, pRadius:Int, pImgOverlay:TImage, pFrame:Int)
		r = pRadius
		x = pX
		y = pY
		img_overlay = pImgOverlay
		img_overlay_frame = pFrame
		list.AddLast(Self)
		Return Self
	End Method

	Function DrawAll()
		SetBlend(LIGHTBLEND)
		For Local o:TLightObstacle = EachIn list
			o.Draw()
		Next
		SetRotation(0)
	End Function
	
	' draws an highlightning overlay on obstacle
	Method Draw()
		' draw hilight
		For Local ls:TLightSource = EachIn TLightSource.list
			Local dx:Float, dy:Float, d:Float, a:Float
			dx = ls.x -x
			dy = ls.y -y
			d = sqr(dx*dx +dy*dy)
			a = atan2(dy,dx)
			If d <= ls.radius then
				Local alpha:Float = 1.0 - d/ls.radius
				glcolor4f ls.color[0], ls.color[1], ls.color[2], alpha
				SetRotation(a)
				DrawImage(img_overlay, x,y, img_overlay_frame)
			EndIf
		Next
	End Method
End Type

Rem ------------------------------------------------------------------
Light geometry describes, where light shines and where it's blocked,
as well as intensities
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Type TLightGeometry
	Field alpha0:Float, alpha1:Float
	Field xc:Float, yc:Float, x0:Float, y0:Float, x1:Float, y1:Float
	Field uc:Float, vc:Float, u0:Float, v0:Float, u1:Float, v1:Float
	Field dx0:Float, dy0:Float, dx1:Float, dy1:Float
	
	' create a fragment of geometry
	Method Init:TLightGeometry(pX:Float, pY:Float, pAngle0:Float, pAngle1:Float, pDist0:Float, pDist1:Float, pAlpha0:Float, pAlpha1:Float)
		xc		= pX
		yc		= pY
		dx0		= cos(pAngle0) *pDist0
		dy0		= sin(pAngle0) *pDist0
		dx1		= cos(pAngle1) *pDist1
		dy1		= sin(pAngle1) *pDist1
		x0		= pX +dx0
		y0		= pY +dy0
		x1		= pX +dx1
		y1		= pY +dy1
		' tex coords (unscaled)
		uc		= pX / LIGHT_STRUCTW
		vc		= pY / LIGHT_STRUCTH
		u0		= dx0 / LIGHT_STRUCTW
		v0		= dy0 / LIGHT_STRUCTH
		u1		= dx1 / LIGHT_STRUCTW
		v1		= dy1 / LIGHT_STRUCTH
		' outer vertex intensity
		alpha0	= pAlpha0
		alpha1	= pAlpha1
		Return Self
	End Method

	' draw a fragment of geometry
	Method Draw(pScale:Float, pColor:Float[], pStrength:Float)
		Local stretch:Float	= 1.0 / pScale
		
		' draw fragment
		gltexcoord2f uc, vc
		glcolor4f pColor[0], pColor[1], pColor[2], pStrength
		glvertex2d xc, yc
		
		gltexcoord2f uc +u0 *stretch, vc +v0 *stretch
		glcolor4f pColor[0], pColor[1], pColor[2], alpha0 *pStrength
		glvertex2d x0, y0

		gltexcoord2f uc +u1 *stretch, vc +v1 *stretch
		glcolor4f pColor[0], pColor[1], pColor[2], alpha1 *pStrength
		glvertex2d x1, y1
	End Method
End Type


Rem ------------------------------------------------------------------
Draw ambient lightning based on given structure.
Light shines in pDirection, pRadius is length of shadow, shadow is
always black, pStrength is alpha.
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Function DrawAmbientLightning(pDirection:Float, pRadius:Float, pStrength:Float)
	SetBlend(ALPHABLEND)
	SetAlpha(pStrength)
	SetColor(0,0,0)
	DrawImage(LIGHT_STRUCTURE, cos(pDirection)*pRadius,sin(pDirection)*pRadius)
	SetBlend(LIGHTBLEND)
	SetAlpha(pStrength)
	SetColor(LIGHT_COL_AMBIENT[0], LIGHT_COL_AMBIENT[1], LIGHT_COL_AMBIENT[2])
	DrawImage(LIGHT_STRUCTURE, 0,0)
End Function


Rem ------------------------------------------------------------------
Return shortest angle between pAFrom and pATo each rangning 0...360.
Returned value ranges -180...180
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Function GetAngleDiff:Float(pAFrom:Float, pATo:Float)
	Local da:Float = pATo -pAFrom
	If da < -180 then da :+ 360
	If da > 180 then da :- 360
	Return da
End Function

Rem ------------------------------------------------------------------
' returns which of both angles is most positive in rotation direction
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Function MostPosAngle:Float(pAngle0:Float, pAngle1:Float)
	If GetAngleDiff(pAngle0, pAngle1)>0 then
		Return pAngle1
	Else
		Return pAngle0
	EndIf
End Function

Rem ------------------------------------------------------------------
' returns which of both angles is most negative in rotation direction
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Function MostNegAngle:Float(pAngle0:Float, pAngle1:Float)
	If GetAngleDiff(pAngle0, pAngle1)<0 then
		Return pAngle1
	Else
		Return pAngle0
	EndIf
End Function
