Rem ------------------------------------------------------------------
Projectiles
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Type TProjectile
	Global list:TList = CreateList()

	Field action:TAction
	Field figure:TFigure
	Field posx:Float, posy:Float
	Field vx:Float, vy:Float
	
	Field color:Int[]
	Field light:TLightSource
	
	' predicted hit
	Field prefigure:TFigure
	Field prepoint:Float[2]
	
	Method Init:TProjectile(pAction:TAction, pFigure:TFigure, pX:Float, pY:Float, pAngle:Float, pOffset:Float, pSpeed:Float, pCol:Int[])
		action = pAction
		figure = pFigure
		posx = pX +cos(pAngle) *pOffset
		posy = pY +sin(pAngle) *pOffset
		vx = cos(pAngle) *pSpeed
		vy = sin(pAngle) *pSpeed
		' light
		color = pCol
		light = New TLightSource.Init(150, color)
		light.x = posx
		light.y = posy
		list.AddLast(Self)
		Return Self
	End Method
	
	Function UpdateAll()
		For Local projectile:TProjectile = EachIn list
			projectile.Update()
			' update prediction for next hit
			projectile.Predict()
		Next
	End Function
	
	Method Update()
		' do in substeps for maximum precision
		Local subs:Int = Int(Max(Abs(vx), Abs(vy))) +1
		Local vxs:Float = vx / subs
		Local vys:Float = vy / subs
		' move and update
		For Local i:Int = 0 Until subs
			posx :+ vxs
			posy :+ vys
			' remove if out of screen
			If posx < -200 Or posx > SCREEN_WIDTH+200 Or posy < -200 Or posy > SCREEN_HEIGHT+100 then
				light.list.Remove(light)
				list.Remove(Self)
				Return
			EndIf
			' check for figure collisions
			Local dx:Float, dy:Float, dist:Float
			For Local fig:TFigure = EachIn TFigure.list
				dx = fig.posx -posx
				dy = fig.posy -posy
				dist = sqr(dx*dx +dy*dy)
				If dist < (fig.radius +3) then
					' visual fx
					For Local i:Int = 0 Until 20
						Local a:Int = Rand(0,359)
						Local r:Float = Rnd(0.2,1)
						New TParticle.Init(posx, posy, cos(a)*r, sin(a)*r, Rnd(0.5, 1), color, Rand(400,600))
					Next
					action.Hit(fig)
					' remove projectile and light and stop updating
					light.list.Remove(light)
					list.Remove(Self)
					Return
				EndIf
			Next
			' gfx
			New TParticle.Init(posx, posy, Rnd(-0.5,0.5),Rnd(-0.5,0.5), 0.5, color, 400)
		Next
		' adapt light position
		light.x = posx
		light.y = posy
	End Method
	
	Function DrawAll()
		For Local projectile:TProjectile = EachIn list
			projectile.Draw()
		Next
	End Function
	
	Method Draw()
		SetBlend(ALPHABLEND)
		SetAlpha(1)
		SetColor(255,255,255)
		DrawImage(imgProjectile, posx, posy)
	End Method
	
	' predicts which target would be hit theoretically
	Method Predict()
		prefigure = GetNearestTouching(posx,posy, vx,vy, 3, prepoint)
	End Method
End Type