Rem ------------------------------------------------------------------
Returns which figure touches the line from pX,pY direction pVX,pVY
with width pWidth. Point of collision will be stored in pHitpoint[].
Does not regard collisions nearer than 1px
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Function GetNearestTouching:TFigure(pX:Float, pY:Float, pVX:Float, pVY:Float, pWidth:Float, pHitpoint:Float[])
	Local nearestdist:Int = -1, nearestfigure:TFigure
	For Local fig:TFigure = EachIn TFigure.list
		' vector to figure
		Local fx:Float, fy:Float
		fx = fig.posx -pX
		fy = fig.posy -pY
		' vector orthogonal to trajectory
		Local ox:Float, oy:Float
		ox = pVY
		oy = -pVX
		' distance to trajectory (dot product)
		Local dist_o:Float
		dist_o = (ox *fx +oy *fy) /sqr(ox*ox +oy*oy)
		' theoretical hit
		If Abs(dist_o) < (fig.radius +pWidth) then
			Local dist_t:Float, norm_v:Float
			norm_v = sqr(pVX*pVX +pVY*pVY)
			dist_t = (pVX *fx +pVY *fy) /norm_v
			' respect only nearest target gets hit
			If dist_t > 0 And (dist_t < nearestdist Or nearestdist = -1) then
				nearestfigure	= fig
				nearestdist		= dist_t
				' theoretical point of impact
				Local hx:Float, hy:Float
				hx = pVX /norm_v *dist_t
				hy = pVY /norm_v *dist_t
				pHitpoint[0] = pX +hx
				pHitpoint[1] = pY +hy
			EndIf
		EndIf
	Next
	Return nearestfigure
End Function

Rem ------------------------------------------------------------------
Normalizes vector given by pX,pY, returns length of initial vector
if length of vector is 0, 0,0 will be the result
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Function Normalize:Float(pX:Float Var, pY:Float Var)
	Local dist:Float
	dist = sqr(pX*pX +pY*pY)
	If dist > 0 then
		pX :/ dist
		pY :/ dist
	EndIf
	Return dist
End Function

Rem ------------------------------------------------------------------
Gauss function.
Returns 1 for pVal = 0, 0.37 for pVal = pHalfwidth63
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Function Gauss:Float(pVal:Float, pHalfwidth63:Float)
	pVal :/ pHalfwidth63
	Return exp(-(pVal*pVal))
End Function

Rem ------------------------------------------------------------------
Inverted ramp function.
Returns 1 for pVal = 0, 0 for pVal >= pBreakPoint
End Rem ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Function InvRamp:Float(pVal:Float, pBreakPoint:Float)
	Return Max(0, 1 -pVal /pBreakpoint)
End Function
